#include "stdafx.h"
#include "scbase.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CSCBase::CSCBase(HANDLE h1,HANDLE h2)
{
	hCom=INVALID_HANDLE_VALUE;
	resetEvent = h1; 
	readEvent = h2;
	
	DWORD threadid;
	//start de thread
	stopCommEvent=CreateEvent(NULL,TRUE,FALSE,NULL);
	readyEvent=CreateEvent(NULL,TRUE,FALSE,NULL);
	stopEvent=CreateEvent(NULL,TRUE,FALSE,NULL);
	commEvent=CreateEvent(NULL,TRUE,FALSE,NULL);
	event1=CreateEvent(NULL,TRUE,FALSE,NULL);
	event2=CreateEvent(NULL,TRUE,FALSE,NULL);
	memset(&overl,0x00,sizeof(OVERLAPPED));
	memset(&overl2,0x00,sizeof(OVERLAPPED));
	memset(&overl3,0x00,sizeof(OVERLAPPED));
	overl.hEvent=commEvent;
	overl2.hEvent=event1;
	overl3.hEvent=event2;
	SetEvent(stopCommEvent);
	thread=CreateThread(NULL,NULL,LocalThread,(void*)this,0,&threadid);
}
CSCBase::~CSCBase()
{
	Close();
	SetEvent(stopEvent);
	WaitForSingleObject(thread,INFINITE);
	CloseHandle(stopEvent);
	CloseHandle(readyEvent);
	CloseHandle(commEvent);
	CloseHandle(stopCommEvent);
	CloseHandle(event1);
	CloseHandle(event2);
}
BYTE CSCBase::adapt(BYTE b)
{
	int i;
	BYTE y,mask,mask2,res;

	y=~b;
	mask=0x01;
	mask2=0x80;
	res=0;
	for(i=0;i<8;i++)
	{
	  if (y & mask)
			res|=mask2;
	  mask<<=1;
	  mask2>>=1;
	}
	return res;
}
void CSCBase::WaitFor()
{
	DWORD	stat;
	
	GetCommModemStatus(hCom,&stat);
	while (stat&MS_RLSD_ON)
	{
		//WaitForSingleObject(stopEvent,1);
		GetCommModemStatus(hCom,&stat);
	}
}
DWORD CSCBase::Read(BYTE *buf,DWORD bytes_to_read,DWORD wait)
{
	DWORD bytes_read,cnt,err;
	BYTE  *buf2;

	cnt=0;
	buf2=buf;
	err=ReadFile(hCom,(void*)buf,bytes_to_read,&bytes_read,&overl2);
	if (!err)
	{
		err=GetLastError();
		ASSERT(err==ERROR_IO_PENDING);
		GetOverlappedResult(hCom,&overl2,&bytes_read,FALSE);
	}
	cnt+=bytes_read;
	while (wait>0 && cnt<bytes_to_read)
	{
		WaitForSingleObject(stopEvent,100);
		buf2+=bytes_read;
		err=ReadFile(hCom,(void*)buf2,bytes_to_read-cnt,&bytes_read,&overl2);
		if (!err)
		{	
			err=GetLastError();
			ASSERT(err==ERROR_IO_PENDING);
			GetOverlappedResult(hCom,&overl2,&bytes_read,FALSE);
		}
		cnt+=bytes_read;
		wait--;
	}
	
	if (invert)
	{
		for(bytes_read=0;bytes_read<cnt;bytes_read++)
			buf[bytes_read]=adapt(buf[bytes_read]);
	}

	return cnt;
}
DWORD CSCBase::Write(BYTE *buf,DWORD bytes_to_write)
{
	DWORD bytes_written=0;
	if (invert)
	{
		for(bytes_written=0;bytes_written<bytes_to_write;bytes_written++)
			loc_buf[bytes_written]=adapt(buf[bytes_written]);
		WriteFile(hCom,(void*)loc_buf,bytes_to_write,&bytes_written,&overl3);
		GetOverlappedResult(hCom,&overl3,&bytes_written,TRUE);
	}
	else
	{
		WriteFile(hCom,(void*)buf,bytes_to_write,&bytes_written,&overl3);
		GetOverlappedResult(hCom,&overl3,&bytes_written,TRUE);
	}
	return bytes_written;
}
void CSCBase::SetConfig(const char *serconfig)
{
	DCB		dcb;
	BOOL	fSuccess;
	DWORD	dwError,err;

	if (hCom==INVALID_HANDLE_VALUE)
		return;
	
	fSuccess = GetCommState(hCom, &dcb);
	if (!fSuccess) 
	{
		AfxMessageBox("GetCommState failed",MB_OK);
		CloseHandle(hCom);
		return;
	}
	fSuccess = BuildCommDCB(serconfig,&dcb);
	if (!fSuccess) 
	{
		AfxMessageBox("BuildCommDCB failed",MB_OK);
		CloseHandle(hCom);
		return;
	}
	dcb.fRtsControl=RTS_CONTROL_DISABLE;
	fSuccess = SetCommState(hCom, &dcb);
	if (!fSuccess) 
	{
		dwError=GetLastError();
		AfxMessageBox("SetCommState failed",MB_OK);
		CloseHandle(hCom);
		return;
	}
	tweak=-1;
	err=WaitCommEvent(hCom,&dwEvtMask,&overl);
	if (!err)
	{
		err=GetLastError();
		ASSERT(err==ERROR_IO_PENDING);
	}
	else
	{
		SetEvent(commEvent);	
	}
	SetEvent(stopCommEvent);
}
void CSCBase::Close()
{
	if (hCom==INVALID_HANDLE_VALUE)
		return;

	SetEvent(stopCommEvent);
	WaitForSingleObject(readyEvent,INFINITE);
	ResetEvent(readyEvent);
	CloseHandle(hCom);
	hCom=INVALID_HANDLE_VALUE;
}
void CSCBase::Open(const char *serconfig)
{
	COMMTIMEOUTS	timeouts;
	char			string[16];
	BOOL	fSuccess;

	if (hCom!=INVALID_HANDLE_VALUE)
		Close();

	memcpy(string,serconfig,4);
	string[4]=0x00;
	hCom = CreateFile(string,GENERIC_READ | GENERIC_WRITE,
					   0, NULL,OPEN_EXISTING, FILE_FLAG_OVERLAPPED,NULL);
	if (hCom == INVALID_HANDLE_VALUE)
	{    
		AfxMessageBox("Could not create handle",MB_OK);
		return;
	}
	
	fSuccess = SetupComm(hCom,1024,1024);
	if (!fSuccess) 
	{
		AfxMessageBox("SetupComm failed",MB_OK);
		CloseHandle(hCom);
		return;
	}
	fSuccess = GetCommTimeouts(hCom,&timeouts);
	if (!fSuccess) 
	{
		AfxMessageBox("GetCommTimeouts failed",MB_OK);
		CloseHandle(hCom);
		return;
	}
	timeouts.ReadIntervalTimeout=MAXDWORD;
	fSuccess = SetCommTimeouts(hCom,&timeouts);
	if (!fSuccess) 
	{
		AfxMessageBox("SetCommTimeouts failed",MB_OK);
		CloseHandle(hCom);
		return;
	}
	fSuccess = SetCommMask(hCom,EV_RLSD|EV_RXCHAR);
	if (!fSuccess) 
	{
		AfxMessageBox("SetCommMask failed",MB_OK);
		CloseHandle(hCom);
		return;
	}
}
void CSCBase::SetInvert(int p)
{
	invert=p;
}
DWORD CSCBase::LocalThread(LPVOID param)
{
	CSCBase*	scbase = (CSCBase*)param;
	DWORD		wobject,wobject2,err,dummy;
	HANDLE		wevents[3] = {NULL,NULL,NULL};
	int	cnt1=0;
	int	cnt2=0;
	
	wevents[0]=scbase->stopEvent;
	wevents[1]=scbase->stopCommEvent;
	wevents[2]=scbase->commEvent;
	

	while ((wobject=WaitForMultipleObjects(3,wevents,FALSE,INFINITE))!=WAIT_FAILED)
	{
		if (wobject==WAIT_OBJECT_0)
		{
			return 0;
		} 
		else if (wobject==WAIT_OBJECT_0+1)
		{
			ResetEvent(scbase->stopCommEvent);
			SetEvent(scbase->readyEvent);
			wobject2=WaitForMultipleObjects(2,wevents,FALSE,INFINITE);
			if (wobject2==WAIT_OBJECT_0)
				return 0;
			ResetEvent(scbase->stopCommEvent);
		}
		else if (wobject==WAIT_OBJECT_0+2)
		{
			
			if (scbase->hCom!=INVALID_HANDLE_VALUE)
			{
				GetOverlappedResult(scbase->hCom,&scbase->overl,&dummy,TRUE);
				if (scbase->dwEvtMask & EV_RLSD)
				{
					scbase->HandleReset();
					cnt1++;
				} 
				else if (scbase->dwEvtMask & EV_RXCHAR)
				{
					scbase->HandleRead();
					cnt2++;
				}
				err=WaitCommEvent(scbase->hCom,&scbase->dwEvtMask,&scbase->overl);
				if (!err)
				{
					err=GetLastError();
					ASSERT(err==ERROR_IO_PENDING);
				}
				else
				{
					SetEvent(wevents[2]);	
				}
			}
			else
				ResetEvent(wevents[2]);
		}
	}
	return 0;
}
void CSCBase::HandleReset()
{
	SetEvent(resetEvent);
}
void CSCBase::HandleRead()
{
	SetEvent(readEvent);
}
void CSCBase::Flush()
{
	if (hCom!=INVALID_HANDLE_VALUE)
		PurgeComm(hCom,PURGE_RXCLEAR);
}